﻿using System;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : Form
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            this.executeButton.Click += executeButton_Click;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 실행 버튼 클릭시 처리하기 - executeButton_Click(sender, e)

        /// <summary>
        /// 실행 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private async void executeButton_Click(object sender, EventArgs e)
        {
            Progress<int> progress = new Progress<int>();

            progress.ProgressChanged += (sender, e) =>
            {
                this.progressBar.Value = e;

                if(e == 100)
                {
                    this.executeButton.Enabled = true;
                }
            };

            this.executeButton.Enabled = false;

            await ProcessJobAsync(progress);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Event

        #region 작업 처리하기 (비동기) - ProcessJobAsync(progress)

        /// <summary>
        /// 작업 처리하기 (비동기)
        /// </summary>
        /// <param name="progress">진행 인터페이스</param>
        /// <returns>태스크</returns>
        private static async Task ProcessJobAsync(IProgress<int> progress)
        {
            int progressRatio = 0;

            for(int i = 0; i < 10; i++)
            {
                // 시간이 걸리는 작업을 실행한다.
                await Task.Delay(TimeSpan.FromMilliseconds(1000));

                progressRatio += 10;

                progress?.Report(progressRatio);
            }
        }

        #endregion
    }
}